@@ -22,6 +22,7 @@ import pickle
 import cPickle
 import pickletools
 import thread
+import threading
 import os.path
 import collections
 import operator
@@ -75,14 +76,60 @@ RANGEITERATOR_TYPE = type(EMPTY_RANGE_ITERATOR)
 EMPTY_SET_ITERATOR = iter(set())
 SETITERATOR_TYPE = type(EMPTY_SET_ITERATOR)
 
-__LOGGER = None
-def LOGGER():
-    global __LOGGER
-    if __LOGGER is None:
-        import logging
-        __LOGGER = logging.getLogger(__name__)
-    return __LOGGER
 
+class _DelayedLogger(object):
+    from imp import lock_held
+    
+    def __init__(self, name):
+        self.__logger = None
+        self.__name = name
+        self.__tl = threading.local()
+    
+    def __getLogger(self):
+        if self.__logger is None:
+            import logging
+            self.__logger = logging.getLogger(self.__name)
+        return self.__logger
+        
+    def __getQueue(self):
+        tl = self.__tl
+        try:
+            return tl.queue
+        except AttributeError:
+            tl.queue = collections.deque()
+        return tl.queue
+        
+    def __call__(self):
+        if self.lock_held():
+            return self
+        self.flush()
+        return self.__getLogger()
+    
+    def __getattr__(self, name):
+        f = functools.partial(self.__log, name)
+        setattr(self, name, f)
+        return f
+    
+    def __log(self, method, *args, **kw):
+        if method is not None:
+            queue = self.__getQueue()
+            queue.append((method, args, kw))
+        
+        if not self.lock_held():
+            queue = self.__getQueue()
+            while True:
+                try:
+                    nextCall = queue.popleft()
+                except IndexError:
+                    break
+                getattr(self.__getLogger(), nextCall[0])(*nextCall[1], **nextCall[2])
+                
+    def flush(self):
+        self.__log(None)
+
+LOGGER = _DelayedLogger(__name__)
+            
+            
 
 # types to be replaced by proxies, if possible  , 
 RESOURCE_TYPES = (file, socket.SocketType, SOCKET_PAIR_TYPE, socket._closedsocket,
@@ -101,7 +148,7 @@ else:
 
 
 #
-# The next 7 functions are used to unpickle modules
+# The next 8 functions are used to unpickle various types
 #
 def create_module(cls, name, doc=None):
     """create and imports a module.
@@ -153,6 +200,13 @@ def restore_modules_entry(doDel, old, new, preserveReferenceToNew=True):
         release_lock()
     return new
 
+#def import_module(name):
+#    global sys
+#    if sys is None:
+#        sys = __import__('sys')
+#    __import__(name)
+#    return sys.modules[name]
+
 def create_thread_lock(locked):
     """recreate a lock object"""
     l = thread.allocate_lock()
@@ -199,6 +253,7 @@ if True:
                       'socket': socket,
                       '__builtins__': __builtins__}   
     __func = type(create_module)
+#    import_module=__func(import_module.func_code, {'sys': None, '__import__': __import__}, 'import_module_')
     create_module=__func(create_module.func_code, __GLOBALS_DICT, 'create_module_')
     save_modules_entry=__func(save_modules_entry.func_code, __GLOBALS_DICT, 'save_modules_entry_')
     restore_modules_entry=__func(restore_modules_entry.func_code, 
@@ -515,6 +570,10 @@ class Pickler(pickle.Pickler):
         return self.__class__.__bases__[0].memoize(self, obj)
         
     def save_dict(self, obj):
+        if obj is sys.modules:
+            self.write(pickle.GLOBAL + 'sys' + '\n' + 'modules' + '\n')
+            self.memoize(obj)
+            return
         self.dict_checkpoint(obj, self._save_dict_impl)
         
     def _save_dict_impl(self, obj):
@@ -674,14 +733,24 @@ class Pickler(pickle.Pickler):
         
         if not self.mustSerialize(obj):
             # help pickling unimported modules
-            if obj.__name__ not in sys.modules:
+            if obj is not sys.modules.get(obj.__name__):
+            # if obj.__name__ not in sys.modules:
                 try:
                     self.save_global(obj)
                     return True
                 except pickle.PicklingError:
-                    pass
-            # continue with the default procedure
-            return False  
+                    LOGGER().exception("Can't pickle anonymous module: %r", obj)
+                    raise
+                
+            # self.save_reduce(import_module, (obj.__name__,), obj=obj)
+            if __import__(obj.__name__) is obj:
+                self.save_reduce(__import__, (obj.__name__,), obj=obj)
+            else:
+                self.save_reduce(__import__, (obj.__name__,))
+                write(pickle.POP)
+                self.save_reduce(operator.getitem, (sys.modules, obj.__name__), obj=obj)
+            return True
+                
         # do it ourself
         
         # save the current implementation of the module
@@ -697,6 +766,8 @@ class Pickler(pickle.Pickler):
             self.save_reduce(create_module, (type(obj), obj.__name__, getattr(obj, "__doc__", None)), obj.__dict__, obj=obj)
         finally:
             restore_modules_entry(True, savedModule, obj, preserveReferenceToNew=False)
+            # Flush log messages
+            LOGGER.flush()
             
         if doDel or not reload:
             write(pickle.TUPLE3+pickle.REDUCE)
